//Example of using blocking, listening, multithreading, graceful shutdown
//Full debug information provided in binary to test for memory leaks using e.g. MemProof
//Connect to 127.0.0.1 port 2876 using telnet.exe to test
//
// http://www.ward.nu/computer/tsock

unit ublock1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Sock, StdCtrls;

type
  TMainForm = class(TForm)
    MasterSock: TSock;
    LogBox: TListBox;
    procedure MasterSockAutoAccept(Sender: TObject; NewSock: TSock);
    procedure MasterSockInfo(sender: TObject; SocketInfo: TSocketInfo;
      Msg: String);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

procedure TMainForm.MasterSockAutoAccept(Sender: TObject; NewSock: TSock);
var
  Command: String;
begin
  //NewSock is automatically created before entering this routine
  LogBox.Items.Insert(0, 'Misc: Remote host is '+NewSock.RemoteHost);
  NewSock.SendLine('Welcome to '+GetLocalHostname+'!');
  while NewSock.Connected do
    try
      Sleep(100);
      Command:=NewSock.ReceiveLine();
      //ReceiveLine returns when:
      // BlockingTimeout expires (return value=''), or
      // data has been received, but no full line (return value=''), or
      // data has been received, full line (return value=full line)
      if Command<>'' then
      begin
        LogBox.Items.Insert(0, 'Receive: '+Command);
        if (UpperCase(Command)='DATE') then
          NewSock.SendLine(DateTimeToStr(Now))
        else if (UpperCase(Command)='QUIT') then
          NewSock.Close
        else
          NewSock.SendLine('Unknown or no command received. Valid commands are: DATE , QUIT');
      end;
    except
      on E:ESockException do LogBox.Items.Insert(0, 'Exception: '+E.Message);
    end;
  //NewSock is automatically destroyed after leaving this routine
end;

procedure TMainForm.MasterSockInfo(Sender: TObject; SocketInfo: TSocketInfo;
  Msg: String);
begin
  LogBox.Items.Insert(0, 'OnInfo: '+Msg);
end;

procedure TMainForm.FormClose(Sender: TObject; var Action: TCloseAction);
var
  I: Integer;
  EnumComp: TComponent;
begin
  //Stop accepting new connections
  MasterSock.StopListen;
  LogBox.Items.Insert(0, 'Misc: Terminating open connections... Please wait...');
  //Loop through all the owned components(=all autoaccepted socks)
  for I:=MasterSock.ComponentCount-1 downto 0 do
  begin
    EnumComp:=MasterSock.Components[I];
    //If it's a TSock, then close it
    //Sending the close command, will at the next iteration stop the
    //TMainForm.MasterSockAutoAccept function because NewSock.Connected will be false
    if (EnumComp is TSock) then
    begin
      try
        (EnumComp as TSock).SendLine('Server shutting down... Bye!');
      except
        on E:ESockException do LogBox.Items.Insert(0, 'Exception: '+E.Message);
      end;
      (EnumComp as TSock).Close;
    end;
  end;
  //Wait a second for all threads to finish
  for I:=0 to 100 do
  begin
    Application.ProcessMessages;
    Sleep(10);
  end;
  //And finally shutdown the application...
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  LogBox.Items.Insert(0, 'Misc: '+WSDescription+' ('+WSSystemStatus+')');
end;

end.
